
#include "G4RunManager.hh"

#include "DetectorConstruction.hh"
#include "G4PhysListFactory.hh"
#include "G4VModularPhysicsList.hh"
#include "PrimaryGeneratorAction.hh"

#include "G4UnitsTable.hh"
#include "G4Neutron.hh"
#include "G4Proton.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleTable.hh"
#include "G4NistManager.hh"
#include "G4HadronicProcessStore.hh"

#include "G4NucleiProperties.hh"
#include "G4NistManager.hh"
#include "G4StableIsotopes.hh"

#include <fstream>
#include <sstream>
#include <string>

#include "QGSP_BIC_HP_NADEKO.hh"

static G4String dir = "txt";
static G4String physName = "QGSP_BIC_HP_NADEKO";
static G4double emin = 1.*MeV;
static G4double emax = 1000.*MeV;
static G4double estep = 1.01;
static G4String energyunit = "MeV";
static G4String crosssectionunit = "barn";

void CrossSection(G4String elementName, G4String particleName);

// argv[1]: physics name
int main(int argc,char** argv)
{
  G4RunManager* runManager = new G4RunManager();
  runManager->SetUserInitialization(new DetectorConstruction());

  G4VModularPhysicsList* modularPhysicsList = 0;
  if(argc==2){
    physName = argv[1];
    G4PhysListFactory physListFactory;
    modularPhysicsList = physListFactory.GetReferencePhysList(physName);
  }else{
    modularPhysicsList = new QGSP_BIC_HP_NADEKO; // manualy created physics class
  }
  if(!modularPhysicsList) return 1;

  runManager->SetUserInitialization(modularPhysicsList);

  runManager->Initialize();

  runManager->SetUserAction(new PrimaryGeneratorAction());

  runManager->BeamOn(1);

  CrossSection("C", "neutron");
  CrossSection("H", "neutron");

  delete runManager;

  return 0;
}

void CrossSection(G4String elementName, G4String particleName)
{
  std::ofstream ofs((dir+"/"+elementName+"_"+particleName+"_"+physName+".txt").data());

  const G4Element* elm = G4NistManager::Instance()->FindOrBuildElement(elementName);
  const G4ParticleDefinition* particle = G4ParticleTable::GetParticleTable()->FindParticle(particleName);

  if(!particle || !elm) {
    G4cout << "HistoManager WARNING Particle or element undefined" << G4endl;
    return;
  }

  G4cout << "### Fill Cross Sections for " << particleName << " of " << elementName << G4endl;

  G4HadronicProcessStore* store = G4HadronicProcessStore::Instance();

  G4double eu = G4UnitDefinition::GetValueOf(energyunit);
  G4double cu = G4UnitDefinition::GetValueOf(crosssectionunit);

  ofs << "# Physics List : " << physName << G4endl;
  ofs << "# Particle : " << particleName << G4endl;
  ofs << "# Element : " << elementName << G4endl;
  ofs << "# Energy(" << energyunit << ") Elastic(" << crosssectionunit << ") Inelastic(" << crosssectionunit << ")" << G4endl;

  for(G4double e = emin; e < emax; e = e*estep){
    G4Material*  fTargetMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");

    G4double elastic = store->GetElasticCrossSectionPerAtom(particle,e,elm,fTargetMaterial);
    G4double inelastic = store->GetInelasticCrossSectionPerAtom(particle,e,elm,fTargetMaterial);

    ofs << e/eu << " " << elastic/cu << " " << inelastic/cu << G4endl;
  }
}
