
#include "DetectorConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

DetectorConstruction::DetectorConstruction()
{
  fLogicTarget = 0;
  fLogicWorld  = 0;

  fTargetMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

DetectorConstruction::~DetectorConstruction()
{;}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Target LV
  G4ThreeVector target_pos(0., 0., 0.);
  G4ThreeVector target_size(100.0*cm, 4.0*cm, 2.0*cm);
  G4Box* solidT = new G4Box("Target",
			    target_size.x()/2.,
			    target_size.y()/2.,
			    target_size.z()/2.);
  fLogicTarget = new G4LogicalVolume(solidT,fTargetMaterial,"Target");
  new G4PVPlacement(0,
		    target_pos,
		    fLogicTarget,
		    "Target",
		    fLogicWorld,
		    false,
		    0);

  // World LV
  G4ThreeVector world_pos(0., 0., 0.) ;  
  G4ThreeVector world_size(1.2*target_size.x(),
			   1.2*target_size.y(),
			   1.2*2.*(target_pos.z()+target_size.z()/2.0));
  G4Box* solidW = new G4Box("World",
			    world_size.x()/2.,
			    world_size.y()/2.,
			    world_size.z()/2.) ;
  fLogicWorld = new G4LogicalVolume(solidW,fWorldMaterial,"World");
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       world_pos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  return world;
}

