
#include "QGSP_BIC_HP_NADEKO.hh"

#include "globals.hh"
#include "G4ProcessManager.hh"
#include "G4ProcessVector.hh"
#include "G4ParticleTypes.hh"
#include "G4ParticleTable.hh"

#include "G4Material.hh"
#include "G4MaterialTable.hh"
#include "G4ios.hh"
#include <iomanip>   

#include "G4DecayPhysics.hh"
#include "G4EmStandardPhysics.hh"
#include "G4EmExtraPhysics.hh"
#include "G4IonInclAblaPhysics.hh"
#include "G4IonBinaryCascadePhysics.hh"
#include "G4QStoppingPhysics.hh"
#include "HadronPhysicsQGSP_INCLXX.hh"
#include "HadronPhysicsQGSP_BIC_HP.hh"
#include "G4HadronElasticPhysics.hh"
#include "G4HadronElasticPhysicsHP.hh"
#include "G4NeutronTrackingCut.hh"

#include "G4DataQuestionaire.hh"

#include "G4WarnPLStatus.hh"

QGSP_BIC_HP_NADEKO::QGSP_BIC_HP_NADEKO(G4int ver, const G4String&) : G4VModularPhysicsList()
{
  G4DataQuestionaire it(photon, neutron, neutronxs);
  G4cout << "<<< Geant4 Physics List simulation engine: QGSP_BIC_HP_NADEKO"<< G4endl;
  G4cout << G4endl;

  defaultCutValue = 0.7*mm;  
  SetVerboseLevel(ver);

  // EM Physics
  RegisterPhysics( new G4EmStandardPhysics(ver) );

  // Synchroton Radiation & GN Physics
  RegisterPhysics( new G4EmExtraPhysics(ver) );

  // Decays
  RegisterPhysics( new G4DecayPhysics(ver) );

//   // Hadron Elastic scattering
//  RegisterPhysics( new G4HadronElasticPhysics(ver) );
  // Hadron Elastic scattering
  RegisterPhysics( new G4HadronElasticPhysicsHP(ver) );

  //  // Hadron Physics
  //  RegisterPhysics( new HadronPhysicsQGSP_INCLXX(ver));
  // Hadron Physics
  RegisterPhysics( new HadronPhysicsQGSP_BIC_HP(ver));

  // Stopping Physics
  RegisterPhysics( new G4QStoppingPhysics(ver) );

  //  // Ion Physics
  //  RegisterPhysics( new G4IonInclAblaPhysics(ver));
  // Ion Physics
  RegisterPhysics( new G4IonBinaryCascadePhysics(ver));

  // Neutron tracking cut
  G4NeutronTrackingCut* neutronTrackingCut = new G4NeutronTrackingCut(ver);
  neutronTrackingCut->SetTimeLimit(1.0*microsecond);
  neutronTrackingCut->SetKineticEnergyLimit(1.0*MeV);
  RegisterPhysics(neutronTrackingCut);
}

QGSP_BIC_HP_NADEKO::~QGSP_BIC_HP_NADEKO()
{;}

void QGSP_BIC_HP_NADEKO::SetCuts()
{
  SetCutsWithDefault();   
  //  G4VUserPhysicsList::DumpCutValuesTable();  
}
